package httpproxy

import (
	"net"
	"strings"
	"testing"
	_ "unsafe" // use for go:linkname
)

//go:linkname parseIPZone net.parseIPZone
func parseIPZone(s string) (net.IP, string)

type config struct{}

// test useProxy implement
func (cfg *config) useProxy(addr string) bool {
	// get host
	host, _, err := net.SplitHostPort(addr)
	if err != nil {
		host = addr
	}

	// IPv6 with "[]"
	if strings.HasPrefix(host, "[") && strings.HasSuffix(host, "]") {
		host = host[1 : len(host)-1]
	}

	// check localhost
	if host == "localhost" {
		return false
	}

	// test parseIPZone
	ip, _ := parseIPZone(host)
	if ip != nil && ip.IsLoopback() {
		return false
	}

	return true
}

func TestUseProxyWithLoopback(t *testing.T) {
	tests := []struct {
		name     string
		host     string
		expected bool
	}{
		{
			name:     "IPv4 loopback",
			host:     "127.0.0.1",
			expected: false,
		},
		{
			name:     "IPv6 loopback",
			host:     "::1",
			expected: false,
		},
		{
			name:     "IPv6 with zone",
			host:     "fe80::1%eth0",
			expected: true,
		},
		{
			name:     "IPv6 with brackets",
			host:     "[::1]",
			expected: false,
		},
		{
			name:     "IPv4 with port",
			host:     "127.0.0.1:8080",
			expected: false,
		},
		{
			name:     "IPv6 with port",
			host:     "[::1]:8080",
			expected: false,
		},
		{
			name:     "Regular domain",
			host:     "example.com",
			expected: true,
		},
		{
			name:     "localhost",
			host:     "localhost",
			expected: false,
		},
	}

	cfg := &config{}
	for _, tt := range tests {
		t.Run(tt.name, func(t *testing.T) {
			result := cfg.useProxy(tt.host)
			if result != tt.expected {
				t.Errorf("useProxy(%q) = %v, want %v", tt.host, result, tt.expected)
			}
		})
	}
}
